/* 
 Set Local Offset, DST, and Sound
 Jed Margolin  12/8/2024
*/  

#include <stdio.h>
#include <string.h>
#include "pico/stdlib.h"
#include "hardware/i2c.h"
#include "hardware/adc.h"
#include "hardware/spi.h"
#include "hardware/uart.h"
#include "hardware/structs/i2c.h"
#include "pico/binary_info.h"
#include <math.h>
#include "bref.h"

// Set Local UTC offset, DST enabled (or not)
void set_reference()
{
   SetCmdMode();    // set for commands
   lcd_byte(0x01);  // clear screen
   domsg(&msg1);   // "Set Reference"
   sleep_ms(2000);
   clear_lcd(0xC0);

   get_local_flash();
   state = 0;    // Now do Local Offset to UTC

   while(1)
   {
      if(irq_flag)   // 4 ms interrupts, debounces and processes the switches
      {
         irq_flag = 0;

         // Do the core 1 main state machine.
         (*state_ptr[state])();  // Use the state number as the index to the list of pointers to the state functions
      }  // end of irq_flag

   } // end while

   return;
}   // end of set_reference()
//==========================================================
void state0()       // Local Offset from UTC
{
   uint16_t n;

   SetCmdMode();     // set for commands
   lcd_byte(0x01);   // clear lcd, both lines

   domsg(&msg4);         // "Local Offset"

// We have local offset in time.utc_offset and dst enable in time.dst_enable from Flash
   sint8_bcd_str(time.utc_offset, nstr);  // convert to string
   lcd_wr_instr(0xC0);  // position
   for(n=0; n<16; n++)
   {
      if(nstr[n] == 0)  {break;}
      lcd_wr_data(nstr[n]);
   }

   if(switch1.press == 1)
   {
      switch1.press = 0;   // increment
      local_wr = 1;
      time.utc_offset++;
      if(time.utc_offset>12)  { time.utc_offset = 12; }
   }

   if(switch2.press == 1)   // decrement
   {
      switch2.press = 0;   // reset the flag
      local_wr = 1;
      time.utc_offset--;
      if(time.utc_offset<-12) { time.utc_offset = -12; }
   }

   if(switch3.press == 1)   // Next
   {
      switch3.press = 0;   // reset the flag
      state = 1;    // Now do DST Enable
   }

   sleep_ms(500);
}
//-----------------------------
void state1()       // DST Observed?
{
   SetCmdMode();     // set for commands
   lcd_byte(0x01);   // clear lcd, both lines
   domsg(&msg5);     // "DST Observed:"

   if(switch1.press == 1)
   {
      switch1.press = 0;   // reset the flag
      local_wr = 1;
      time.dst_observed = 1;
   }

   if(switch2.press == 1)   // Not updated
   {
      switch2.press = 0;   // reset the flag
      local_wr = 1;
      time.dst_observed = 0;
   }

   if(time.dst_observed)  { domsg(&msg6); }    // "Yes"
   else  { domsg(&msg7); }    // "No "

   if(switch3.press == 1)   // Next
   {
      switch3.press = 0;   // reset the flag
      state = 2;    // Now do Update or Not
   }

   sleep_ms(500);
}
//-----------------------------
void state2()       // Alarm Only?
{
   SetCmdMode();     // set for commands
   lcd_byte(0x01);   // clear lcd, both lines
   domsg(&msg9);     // "Alarm Only:"

   if(switch1.press == 1)
   {
      switch1.press = 0;   // reset the flag
      local_wr = 1;
      time.alarm_only = 1;
   }

   if(switch2.press == 1)   // Not updated
   {
      switch2.press = 0;   // reset the flag
      local_wr = 1;
      time.alarm_only = 0;
   }

   if(time.alarm_only)  { domsg(&msg6); }    // "Yes"
   else  { domsg(&msg7); }    // "No "

   if(switch3.press == 1)   // Next
   {
      switch3.press = 0;   // reset the flag
      state = 3;    // Now do Update or Not
   }

   sleep_ms(500);
}
//-----------------------------
void state3()       // Local 12/24
{
   SetCmdMode();     // set for commands
   lcd_byte(0x01);   // clear lcd, both lines
   domsg(&msg10);    // "Local 12/24"};
   state = 4;
}
//-----------------------------
void state4()       // Local 12/24
{
   if(time.hours12_flag)  {domsg(&msg11);}  // 12
   else         {domsg(&msg12);}  // 24

   if(switch1.press == 1)
   {
      time.hours12_flag = 0x01;
      switch1.press = 0;   // reset the flag
   }

   if(switch2.press == 1)
   {
      time.hours12_flag = 0x00;
      switch2.press = 0;   // reset the flag
   }

   if(switch3.press == 1)
   {
      switch3.press = 0;   // next
      state = 5;
   }
} 
//-----------------------------
void state5()       // Save?
{
   SetCmdMode();     // set for commands
   lcd_byte(0x01);   // clear lcd, both lines
   domsg(&msg8);     // "Save?  N/Y"

   if(switch1.press == 1)  // Yes
   {
      switch1.press = 0;   // reset the flag

      do_local(time.utc_offset, time.dst_observed, time.alarm_only, time.hours12_flag, 1);   // erase

      SetCmdMode();     // set for commands
      domsg(&msg2);  // "Updated"
      sleep_ms(1000);
      state = 6;
   }

   if(switch2.press == 1)   // Not updated
   {
      switch2.press = 0;   // reset the flag

      SetCmdMode();     // set for commands
      domsg(&msg3);  // "Ok, not changed"
      sleep_ms(1000);
      local_wr = 0;
      state = 6;
   }

   if(switch3.press == 1)   // Next
   {
      switch3.press = 0;   // reset the flag
      sleep_ms(500);
      state = 4;
   }

   sleep_ms(500);
}
//-----------------------------
void state6()       // Start over
{
   SetCmdMode();     // set for commands
   lcd_byte(0x01);   // clear lcd, both lines
   state = 0;
}
//============================================================
