// Jed Margolin  12/8/2024
// Test core 1

#include <stdio.h>
#include "pico/stdlib.h"
#include "pico/multicore.h"
#include "hardware/pwm.h"
#include "hardware/irq.h"
#include <string.h>
#include "core1.h"   // mine
//=====================================
// Make the sine wave
bool repeating_timer_callback1(struct repeating_timer *t) 
{
   if(!mute) 
   { 
      pwm_set_chan_level(slice_num, PWM_CHAN_A, SINE64[sine_index]);
      sine_index++;
      sine_index &= 0x003F;  // 63 to wrap the sine table at 64 segments

      //   pwm_set_chan_level(slice_num, PWM_CHAN_A, SINE32[sine_index]);
      //   sine_index++;
      //   sine_index &= 0x001F;  // 63 to wrap the sine table at 64 segments
   }

   return true;
}
//=====================================
void core1_entry() 
{   
   sleep_ms(200);
   multicore_fifo_push_blocking(FLAG_VALUE);      // send to core 0
   sleep_ms(200);
   uint32_t g = multicore_fifo_pop_blocking();    // core 0 should send it back
   if (g != FLAG_VALUE)  { uart_puts(UART_ID,"\nIt didn't go right on core 1\n\n"); }
   else                  { uart_puts(UART_ID,"\nGood on core 1\n"); }

   sleep_ms(100);
   uart_puts(UART_ID,"\nCore 1 Here\n\n");

   uart_puts(UART_ID,"Core FIFO uses an irq\n\n");
//---------------------------
   multicore_fifo_clear_irq();
   irq_set_exclusive_handler(SIO_IRQ_PROC1, core1_sio_irq);   // put the FIFO interrupt function here
   irq_set_enabled(SIO_IRQ_PROC1, true);
//---------------------------
// set up pwm
   sine_index = 0;
   mute = 0;
   sleep_ms(1000);
   blink1_led(3, 300, 500);  // blink LED 3 times, 300ms On, 500ms Off

   // Tell GPIO 16 it is allocated to the PWM
   gpio_set_function(16, GPIO_FUNC_PWM);

    // Find out which PWM slice is connected to GPIO 16
   slice_num = pwm_gpio_to_slice_num(16);

   // Set period of 2048 cycles (0 to 2047)
   pwm_set_wrap(slice_num, 2048);  

   // Set channel A output high for 2047 cycles before dropping
   pwm_set_chan_level(slice_num, PWM_CHAN_A, 2048);

   // Set initial B output high for 2048 cycles before dropping
   // pwm_set_chan_level(slice_num, PWM_CHAN_B, 2048);
 
   // Set the PWM running
    pwm_set_enabled(slice_num, true);
   // end setup_pwm[]

   sleep_ms(4);
   new_note = 0;

//   do_tone1();    // test: Doesn't come back
//   do_tone2();    // test: Doesn't come back -  Alarm

// now do it
   while(1)
   {
      while(alarm) 
      { 
         add_repeating_timer_us(note3,repeating_timer_callback1, NULL, &timer1);  
         sleep_ms(500);
         cancel_repeating_timer(&timer1);

         add_repeating_timer_us(note1,repeating_timer_callback1, NULL, &timer1);  
         sleep_ms(500);
         cancel_repeating_timer(&timer1);
      }

      if(new_note)
      {
         new_note = 0;

         add_repeating_timer_us(note,repeating_timer_callback1, NULL, &timer1);  
         sleep_ms(note_time);
         cancel_repeating_timer(&timer1);
      }
   }
}
//====================================================
// D0 - D5 is seconds, D6 is mute except for alarm, D7 is alarm 
void core1_sio_irq()            // FIFO interrupt function
{
   multicore_fifo_clear_irq();

// get seconds from core 0 
   if(multicore_fifo_rvalid())
   {
      core1_rx_val = multicore_fifo_pop_blocking();

      seconds = core1_rx_val & 0x3F;   // 0 - 63 mask
      alarm = core1_rx_val & 0x80;

      mute = core1_rx_val & 0x40;
      if(alarm)  { mute = 0;}
//-------------
      do_clock8_1(seconds);   // convert hex byte to string
      strcpy(line1, nstr1);
      
// Don't use UART here if you are using it in core 0
/*
      if( (seconds == 29) | (seconds == 59) )  { strcat(line1,"\n"); }
      else                                     { strcat(line1, "  "); }
      if( seconds == 59) { strcat(line1,"\n"); } 
      uart_puts(UART_ID,line1);
*/ 
      if(old_seconds != seconds)
      {
         old_seconds = seconds;
         new_note = 1;
         switch(seconds)
         {
            case 57 :
               note = note1; 
               note_time = 900;
               break;

            case 58 :
               note = note2;  // E
               note_time = 900;
               break;

            case 59 :
               note = note3;  // E
               note_time = 900;
               break;

            case 0 :
               note = note4;  // E
               note_time = 80;
               break;

            case 30 :
               note = note4;  // E
               note_time = 80;
               break;

            default :
               note = note1;  // E
               note_time = 80;
               break;
         }  // end of switch(seconds)

      }  // end of new second
   }

   return;
}  // end of function
//==========================================
#if NO
// Tell compiler not to optimize - This works 151 ns
#pragma GCC push_options
#pragma GCC optimize ("O0")
void mywait1()
{
   asm ("nop");
   asm ("nop");
   asm ("nop");
}
#pragma GCC pop_options
#endif
//===================================================
void do_tone1()
{
   uint16_t n;

   while(1)
   {
      cancel_repeating_timer(&timer1);
      add_repeating_timer_us(note1,repeating_timer_callback1, NULL, &timer1);  // C

      sleep_ms(1000);

      cancel_repeating_timer(&timer1);
      add_repeating_timer_us(note2,repeating_timer_callback1, NULL, &timer1);  // E

      sleep_ms(1000);

      cancel_repeating_timer(&timer1);
      add_repeating_timer_us(note3,repeating_timer_callback1, NULL, &timer1);  // G

      sleep_ms(1000);

      cancel_repeating_timer(&timer1);
      add_repeating_timer_us(note4,repeating_timer_callback1, NULL, &timer1);  // C

      sleep_ms(150);
      cancel_repeating_timer(&timer1);
      sleep_ms(2000);

      for(n=0; n<4; n++)
      {
         cancel_repeating_timer(&timer1);
         add_repeating_timer_us(note5,repeating_timer_callback1, NULL, &timer1);  

         sleep_ms(75);
         cancel_repeating_timer(&timer1);
         sleep_ms(975);
      }
      sleep_ms(2000);

      for(n=0; n<4; n++)
      {
         cancel_repeating_timer(&timer1);
         add_repeating_timer_us(note3,repeating_timer_callback1, NULL, &timer1);  
         sleep_ms(500);

         cancel_repeating_timer(&timer1);
         add_repeating_timer_us(note1,repeating_timer_callback1, NULL, &timer1);  
         sleep_ms(500);
      }

      cancel_repeating_timer(&timer1);
      sleep_ms(2000);
   }
}  // end main
//===================================================
void do_tone2()
{
   while(1)
   {
      cancel_repeating_timer(&timer1);
      add_repeating_timer_us(note3,repeating_timer_callback1, NULL, &timer1);  // C

      sleep_ms(500);

      cancel_repeating_timer(&timer1);
      add_repeating_timer_us(note1,repeating_timer_callback1, NULL, &timer1);  // E

      sleep_ms(500);
   }
}  // end main
//===================================================
void blink1_led(unsigned char m, unsigned int time1, unsigned int time2)
{
   unsigned char n;

   for(n=0; n<m; n++)
   {
      gpio_put(LED2,1);
      sleep_ms(time1);
      gpio_put(LED2,0);
      sleep_ms(time2);
   }
}
//===================================================
// for the clock convert hex to ASCII string
void do_clock8_1(uint8_t value)
{
   uint16_t n;

   n = value<<1;

   nstr1[0] = clock8_1[n++];
   nstr1[1] = clock8_1[n++];
   nstr1[2] = 0;  // NULL
}
//======================================================
