// core1.h
#include "common.h"

void do_tone1(void);
void do_tone2(void);

void core1_entry(void);   // core 1 main
void core1_sio_irq(void);

uint32_t core1_rx_val;
uint16_t seconds, old_seconds, new_note, note_time, alarm, mute;
int16_t note;

char line1[256];
char nstr1[256];

struct repeating_timer timer1;

unsigned int slice_num; 

void blink1_led(unsigned char, unsigned int, unsigned int);

const unsigned int SINE64[64] = 
{ 1024, 1122, 1219, 1314, 1407, 1495, 1580, 1658,
  1731, 1797, 1855, 1906, 1948, 1981, 2005, 2019,
  2024, 2019, 2005, 1981, 1948, 1906, 1855, 1797,
  1731, 1658, 1580, 1495, 1407, 1314, 1219, 1122,
  1024,  926,  829,  734,  641,  553,  468,  390,
   317,  251,  193,  142,  100,   67,   43,   29,
    24,   29,   43,   67,  100,  142,  193,   251,
   317,  390,  468,  553,  641,  734,  829,   926 };

unsigned char sine_index;
const unsigned int SINE32[32] = 
{ 2048, 2438, 2813, 3159, 
  3462, 3711, 3896, 4010,
  4048, 4010, 3896, 3711,
  3462, 3159, 2813, 2438,
  2048, 1658, 1283,  937,
   634,  385,  200,   86,
    48,   86,  200,  385,
   634,  937, 1283, 1658 };

// C Major cord
const int16_t note1 = -48;  // C  523 Hz
const int16_t note2 = -38;  // E  659 Hz
const int16_t note3 = -32;  // G  784 Hz
const int16_t note4 = -24;  // C  1046 Hz
const int16_t note5 = -48;  // C  523 Hz

const int16_t NOTES[5] = { note1, note2, note3, note4, note5};

void do_clock8_1(uint8_t);
// Lookup table for converting hex to ASCII, clock only needs to go up to 60 (60 for possible leap seconds)
uint8_t clock8_1[122] = 
{
//     00          01          02          03           04         05          06          07          08          09          
0x30, 0x30, 0x30, 0x31, 0x30, 0x32, 0x30, 0x33, 0x30, 0x34, 0x30, 0x35, 0x30, 0x36, 0x30, 0x37, 0x30, 0x38, 0x30, 0x39,

//     10          11          12          13           14         15          16          17          18          19          
0x31, 0x30, 0x31, 0x31, 0x31, 0x32, 0x31, 0x33, 0x31, 0x34, 0x31, 0x35, 0x31, 0x36, 0x31, 0x37, 0x31, 0x38, 0x31, 0x39,

//     20          21          22          23           24         25          26          27          28          29          
0x32, 0x30, 0x32, 0x31, 0x32, 0x32, 0x32, 0x33, 0x32, 0x34, 0x32, 0x35, 0x32, 0x36, 0x32, 0x37, 0x32, 0x38, 0x32, 0x39,

//     30          31          32          33           34         35          36          37          38          39          
0x33, 0x30, 0x33, 0x31, 0x33, 0x32, 0x33, 0x33, 0x33, 0x34, 0x33, 0x35, 0x33, 0x36, 0x33, 0x37, 0x33, 0x38, 0x33, 0x39,

//     40          41          42          43           44         45          46          47          48          49          
0x34, 0x30, 0x34, 0x31, 0x34, 0x32, 0x34, 0x33, 0x34, 0x34, 0x34, 0x35, 0x34, 0x36, 0x34, 0x37, 0x34, 0x38, 0x34, 0x39,

//     50          51          52          53           54         55          56          57          58          59          
0x35, 0x30, 0x35, 0x31, 0x35, 0x32, 0x35, 0x33, 0x35, 0x34, 0x35, 0x35, 0x35, 0x36, 0x35, 0x37, 0x35, 0x38, 0x35, 0x39,

//     60
0x36, 0x30
};
//=============================================
// compiler does not optimize this; not used here
void mywait1(void);
//==============================================================
/*
Functions for the inter-core FIFOs.
The RP2040 contains two FIFOs for passing data, messages or ordered events between the two cores. Each FIFO is 32 bits wide, and 8 entries deep. One of the FIFOs can only be written by core 0, and read by core 1. The other can only be written by core 1, and read by core 0.
---------------
multicore_fifo_rvalid()
static bool multicore_fifo_rvalid(void )    // inlinestatic

Check the read FIFO to see if there is data available (sent by the other core)
Returns true if the FIFO has data in it, false otherwise
---------------
multicore_fifo_pop_blocking()
uint32_t multicore_fifo_pop_blocking(void)          

Pop data from the read FIFO (data from the other core).
This function will block until there is data ready to be read.
Use multicore_fifo_rvalid() to check if data is ready to be read if you don't want to block.
Returns 32 bit data from the read FIFO.
---------------
multicore_fifo_pop_timeout_us()
bool multicore_fifo_pop_timeout_us(uint64_t timeout_us,  uint32_t *out )                      

Pop data from the read FIFO (data from the other core) with timeout.
This function will block until there is data ready to be read or the timeout is reached

Parameters
timeout_us      the timeout in microseconds
Out             the location to store the popped data if available

Returns true if the data was popped and a value copied into out, false if the timeout occurred before data could be popped
---------------
multicore_fifo_wready()
static bool multicore_fifo_wready(void)      //  inlinestatic

Check the write FIFO to see if it has space for more data.
Returns true if the FIFO has room for more data, false otherwise
---------------
multicore_fifo_clear_irq()
static void multicore_fifo_clear_irq(void)     //  inlinestatic

Clear FIFO interrupt.
Note that this only clears an interrupt that was caused by the ROE or WOF flags. To clear the VLD flag you need to use one of the 'pop' or 'drain' functions.
---------------
multicore_fifo_drain()
static void multicore_fifo_drain(void)   // inlinestatic

Discard any data in the read FIFO.
---------------
multicore_fifo_get_status()
static uint32_t multicore_fifo_get_status(void)       // inlinestatic

Get FIFO statuses.

Returns The statuses as a bitfield
Bit       Description
3          Sticky flag indicating the RX FIFO was read when empty (ROE). This read was ignored by the FIFO.
2          Sticky flag indicating the TX FIFO was written when full (WOF). This write was ignored by the FIFO.
1          Value is 1 if this core's TX FIFO is not full (i.e. if FIFO_WR is ready for more data)
0          Value is 1 if this core's RX FIFO is not empty (i.e. if FIFO_RD is valid)
---------------
multicore_fifo_push_blocking()
void multicore_fifo_push_blocking(uint32_t data)

Push data on to the write FIFO (data to the other core).

This function will block until there is space for the data to be sent. Use multicore_fifo_wready() to check if it is possible to write to the FIFO if you don't want to block.

Parameters
data     A 32 bit value to push on to the FIFO
---------------
multicore_fifo_push_timeout_us()
bool multicore_fifo_push_timeout_us(uint32_t data, uint64_t  timeout_us)

Push data on to the write FIFO (data to the other core) with timeout.
This function will block until there is space for the data to be sent or the timeout is reached

Parameters
data     A 32 bit value to push on to the FIFO
timeout_us      the timeout in microseconds

Returns true if the data was pushed, false if the timeout occurred before data could be pushed
---------------
irq_set_exclusive_handler(SIO_IRQ_PROC1, core1_sio_irq);   // put the FIFO interrupt function here
 
irq_set_enabled(SIO_IRQ_PROC1, true);   // FIFO irq has lower priority than pwm
*/
//&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
